indexing

	description: "Class that covers Posix semaphore routines."

	author: "Berend de Boer"
	date: "$Date: 2003/03/20 $"
	revision: "$Revision: #3 $"

class

	POSIX_NAMED_SEMAPHORE


inherit

	POSIX_SEMAPHORE
		export
			{ANY} SEM_VALUE_MAX
		end


creation

	open,
	create_nonexclusive,
	create_exclusive,
	create_mode


feature {NONE} -- creation

	open (a_name: STRING) is
			-- opens a named semaphore
		require
			uninitialized: not is_initialized
			semaphores_supported: supports_semaphores
			valid_name: a_name /= Void and then not a_name.is_empty
		do
			name := a_name
			sem := posix_sem_open (sh.string_to_pointer (name), 0, 0, 0)
			sh.unfreeze_all
		ensure
			initialized: is_initialized
		end

	create_nonexclusive (a_name: STRING; a_value: INTEGER) is
			-- creates a named semaphore
		require
			uninitialized: not is_initialized
			semaphores_supported: supports_semaphores
			valid_name: a_name /= Void and then not a_name.is_empty
			valid_value: a_value >= 0 and a_value <= SEM_VALUE_MAX
		do
			create_mode (a_name, False, S_IRUSR + S_IWUSR, a_value)
		end

	create_exclusive (a_name: STRING; a_value: INTEGER) is
			-- creates a named semaphore
		require
			uninitialized: not is_initialized
			semaphores_supported: supports_semaphores
			valid_name: a_name /= Void and then not a_name.is_empty
			valid_value: a_value >= 0 and a_value <= SEM_VALUE_MAX
		do
			create_mode (a_name, False, S_IRUSR + S_IWUSR, a_value)
		end

	create_mode (a_name: STRING; exclusive: BOOLEAN; a_mode: INTEGER; a_value: INTEGER) is
			-- creates a named semaphore with permissions given by
			-- `a_mode' and initial value `a_value'.
		require
			uninitialized: not is_initialized
			semaphores_supported: supports_semaphores
			valid_name: a_name /= Void and then not a_name.is_empty
			valid_value: a_value >= 0 and a_value <= SEM_VALUE_MAX
		local
			oflag: INTEGER
		do
			name := a_name
			oflag := O_CREAT
			if exclusive then
				oflag := oflag + O_EXCL
			end
			sem := posix_sem_open (sh.string_to_pointer (a_name), oflag, a_mode, a_value)
			sh.unfreeze_all
			if sem = SEM_FAILED then
				raise_posix_error
			end
		ensure
			initialized: is_initialized
		end


feature -- state

	name: STRING


end
