indexing

	description: "ISE conversion of strings to pointers and vice versa."
	thanks: "The mico/E team for the idea."

	date: "$Date: 2003/11/29 $"
	revision: "$Revision: #8 $"

class

	EPX_STRING_HELPER


inherit

	ABSTRACT_STRING_HELPER
		redefine
			wipe_out,
			unfreeze_all
		end

	EPX_ISE_GC_HELPER


feature -- ISE specific conversions

	pointer_to_string (p: POINTER): STRING is
		do
			if p = default_pointer then
				Result := ""
			else
				!! Result.make_from_c (p)
			end
		end

	set_string_from_pointer (s: STRING; p: POINTER) is
		do
			if p = default_pointer then
				s.clear_all
			else
				s.make_from_c (p)
			end
		end

	unfreeze_all is
		do
			precursor
			unfreeze_objects
		end


feature -- String utilities that have different names per compiler...

	wipe_out (s: STRING) is
			-- Remove all characters in `s', but do not change its
			-- capacity. Notably ISE doesn't implement ELKS correctly
			-- here.
		do
			s.clear_all
		end


feature {NONE} -- Implementation

	clear_unsaved_strings is
			-- The strings in `saved_strings' must be released to the
			-- garbage collector upon `unfreeze_all'.
		do
			saved_strings.clear_all
			saved_strings.make (0, -1)
		end

	do_string_to_pointer (s: STRING): POINTER is
			-- Return a pointer to a linear area containing all the data
			-- in `s'. The area is zero-terminated.
			-- Pointers returned by this routine should only be used
			-- inside a single routine. They should not be kept alive
			-- longer than the duration of that single routine, and that
			-- single routine may not call any other routine which calls
			-- this function. Inside eposix this is guaranteed, but you
			-- are on your own if you decide to call this routine
			-- yourself.
			-- For moving garbage collectors you should regularly call
			-- `unfreeze_all', else memory will not be freed.
		local
			a: ANY
		do
			a := s.to_c
			if a /= Void then
				-- When the garbage collection is disabled, $a just works.
				-- And because the string we pass to the C function
				-- must be short-lived, enabling the gc after the call
				-- is made, should not pose any problems.
				if memory.collecting then
					Result := eif_freeze (a)
					store_freezed_object (Result)
				else
					Result := $a
				end
			end
		ensure then
			one_more_freezed_object:
				not memory.collecting or else
				highest_freezed_object.item = old highest_freezed_object.item + 1
			this_is_last_freezed_pointer:
				not memory.collecting or else
				freezed_objects.item (highest_freezed_object.item) = Result
		end

	do_uc_string_to_pointer (s: UC_STRING): POINTER is
			-- Return a pointer to a linear area containing all the data
			-- in the Unicode string `s'.
			-- Currently getting such a pointer is not an operation
			-- directly supported by UC_STRING and we need to do
			-- different things for certain compilers.
		local
			real_s: STRING
		do
			if s.count = s.byte_count then
				Result := do_string_to_pointer (s)
			else
				real_s := s.as_string
				Result := do_string_to_pointer (real_s)
				-- A big issue now is to keep `real_s' alive until the next
				-- `unfreeze_all'.
				saved_strings.force (real_s, saved_strings.upper + 1)
			end
		end


invariant

	highest_freezed_object_remains_valid: highest_freezed_object >= 0 and then highest_freezed_object <= freezed_objects.upper
	freezed_objects_lower_is_one: freezed_objects.lower = 1

end
